@@ -17,6 +17,20 @@ test_that("sample respects weight", {
   expect_equal(sample_frac(df, 0.5, weight = y)$x, 2)
 })
 
+test_that("slice does not evaluate the expression in empty groups (#1438)", {
+  res <- mtcars %>%
+    group_by(cyl) %>%
+    filter(cyl==6) %>%
+    slice(1:2)
+  expect_equal(nrow(res), 2L)
+
+  expect_error(
+    res <- mtcars %>% group_by(cyl) %>% filter(cyl==6) %>% sample_n(size=3),
+    NA
+  )
+  expect_equal(nrow(res), 3L)
+})
+
 # Grouped ----------------------------------------------------------------------
 
 test_that("sampling grouped tbl samples each group", {
@@ -86,34 +100,50 @@ test_that("sample_n and sample_frac handles lazy grouped data frames (#3380)", {
 # Errors --------------------------------------------
 
 test_that("sample_*() gives meaningful error messages", {
-  df2 <- tibble(
-    x = rep(1:2, 100),
-    y = rep(c(0, 1), 100),
-    g = rep(1:2, each = 100)
-  )
+  expect_snapshot({
+    df2 <- tibble(
+      x = rep(1:2, 100),
+      y = rep(c(0, 1), 100),
+      g = rep(1:2, each = 100)
+    )
 
-  grp <- df2 %>% group_by(g)
+    grp <- df2 %>% group_by(g)
 
-  # base R error messages
-  expect_snapshot(error = TRUE, sample_n(grp, nrow(df2) / 2, weight = y))
-  expect_snapshot(error = TRUE, sample_frac(grp, 1, weight = y))
+    # base R error messages
+    (expect_error(
+      sample_n(grp, nrow(df2) / 2, weight = y)
+    ))
+    (expect_error(
+      sample_frac(grp, 1, weight = y)
+    ))
 
-  # can't sample more values than obs (without replacement)
-  expect_snapshot(error = TRUE, mtcars %>% group_by(cyl) %>% sample_n(10))
+    # can't sample more values than obs (without replacement)
+    (expect_error(
+      mtcars %>% group_by(cyl) %>% sample_n(10)
+    ))
 
-  # unknown type
-  expect_snapshot(error = TRUE, sample_n(list()))
-  expect_snapshot(error = TRUE, sample_frac(list()))
+    # unknown type
+    (expect_error(
+      sample_n(list())
+    ))
+    (expect_error(
+      sample_frac(list())
+    ))
 
-  # helper function check_weight()
-  expect_snapshot(error = TRUE, check_weight(letters[1:2], 2))
-  expect_snapshot(error = TRUE, check_weight(-1:-2, 2))
-  expect_snapshot(error = TRUE, check_weight(letters, 2))
+    "# respects weight"
+    df <- data.frame(x = 1:2, y = c(0, 1))
+    (expect_error(
+      sample_n(df, 2, weight = y)
+    ))
+    (expect_error(
+      sample_frac(df, 2)
+    ))
+    (expect_error(
+      sample_frac(df %>% group_by(y), 2)
+    ))
+    (expect_error(
+      sample_frac(df, 1, weight = y)
+    ))
+  })
 
-  "# respects weight"
-  df <- data.frame(x = 1:2, y = c(0, 1))
-  expect_snapshot(error = TRUE, sample_n(df, 2, weight = y))
-  expect_snapshot(error = TRUE, sample_frac(df, 2))
-  expect_snapshot(error = TRUE, sample_frac(df %>% group_by(y), 2))
-  expect_snapshot(error = TRUE, sample_frac(df, 1, weight = y))
 })
